"""
MiniCC 配置管理

处理 ~/.minicc 目录下的配置文件与系统提示词（AGENTS.md）。
"""

from __future__ import annotations

import os
from pathlib import Path

from .models import Config, Provider

# 配置文件路径
CONFIG_DIR = Path.home() / ".minicc"
CONFIG_FILE = CONFIG_DIR / "config.json"
AGENTS_FILE = CONFIG_DIR / "AGENTS.md"
MCP_CONFIG_FILE = CONFIG_DIR / "mcp.json"

# 项目级 MCP 配置位置：<cwd>/.minicc/mcp.json
PROJECT_CONFIG_DIRNAME = ".minicc"
PROJECT_MCP_CONFIG_NAME = "mcp.json"

# 内置系统提示词文件路径
BUILTIN_PROMPT_FILE = Path(__file__).resolve().parent.parent / "prompts" / "system.md"


def _load_builtin_prompt() -> str:
    if BUILTIN_PROMPT_FILE.exists():
        return BUILTIN_PROMPT_FILE.read_text(encoding="utf-8")
    return "你是一个代码助手，帮助用户完成编程任务。"


def ensure_config_dir() -> None:
    CONFIG_DIR.mkdir(parents=True, exist_ok=True)
    if not CONFIG_FILE.exists():
        save_config(Config())
    if not AGENTS_FILE.exists():
        AGENTS_FILE.write_text(_load_builtin_prompt(), encoding="utf-8")


def load_config() -> Config:
    ensure_config_dir()
    if CONFIG_FILE.exists():
        content = CONFIG_FILE.read_text(encoding="utf-8")
        return Config.model_validate_json(content)
    return Config()


def save_config(config: Config) -> None:
    CONFIG_DIR.mkdir(parents=True, exist_ok=True)
    CONFIG_FILE.write_text(config.model_dump_json(indent=2), encoding="utf-8")


def load_agents_prompt() -> str:
    ensure_config_dir()
    if AGENTS_FILE.exists():
        return AGENTS_FILE.read_text(encoding="utf-8")
    return _load_builtin_prompt()


def get_api_key(provider: Provider) -> str:
    config = load_config()
    if config.api_key:
        return config.api_key

    env_var_map = {
        Provider.ANTHROPIC: "ANTHROPIC_API_KEY",
        Provider.OPENAI: "OPENAI_API_KEY",
    }
    env_var = env_var_map.get(provider)
    if env_var:
        api_key = os.environ.get(env_var)
        if api_key:
            return api_key
    raise ValueError(
        f"未找到 {provider.value} 的 API 密钥。"
        f"请设置环境变量 {env_var} 或在 ~/.minicc/config.json 中配置 api_key"
    )


def find_mcp_config(cwd: str | Path | None = None) -> Path | None:
    base = Path(cwd) if cwd is not None else Path(os.getcwd())
    project_path = base / PROJECT_CONFIG_DIRNAME / PROJECT_MCP_CONFIG_NAME
    if project_path.exists():
        return project_path
    if MCP_CONFIG_FILE.exists():
        return MCP_CONFIG_FILE
    return None




from __future__ import annotations

import asyncio
from dataclasses import dataclass

import pytest

from minicc.core.events import EventBus, SubAgentCreated, SubAgentUpdated
from minicc.core.models import Config, MiniCCDeps, Provider
from minicc.core.services.subagents import SubAgentService


@dataclass
class _DummyResult:
    output: str


class _DummyAgent:
    def __init__(self, output: str, delay: float = 0.0):
        self._output = output
        self._delay = delay

    async def run(self, prompt: str, deps=None):
        if self._delay:
            await asyncio.sleep(self._delay)
        return _DummyResult(output=f"{self._output}:{prompt}")


def _make_deps() -> MiniCCDeps:
    cfg = Config(provider=Provider.ANTHROPIC, model="test-model", api_key="test-key")
    return MiniCCDeps(config=cfg, cwd="/tmp/minicc-test", fs=None)


@pytest.mark.asyncio
async def test_subagent_service_foreground_emits_events_and_returns_result():
    deps = _make_deps()
    bus: EventBus = EventBus()

    service = SubAgentService(
        deps=deps,
        event_bus=bus,
        agent_factory=lambda: _DummyAgent("ok"),
    )

    task_id, result = await service.run(prompt="P", description="D", background=False)
    assert result == "ok:P"
    assert deps.sub_agents[task_id].status == "completed"
    assert deps.sub_agents[task_id].result == "ok:P"

    ev1 = await asyncio.wait_for(bus.next(), timeout=1)
    ev2 = await asyncio.wait_for(bus.next(), timeout=1)
    ev3 = await asyncio.wait_for(bus.next(), timeout=1)
    ev4 = await asyncio.wait_for(bus.next(), timeout=1)

    assert isinstance(ev1, SubAgentCreated)
    assert ev1.task_id == task_id
    assert isinstance(ev2, SubAgentUpdated) and ev2.status == "pending"
    assert isinstance(ev3, SubAgentUpdated) and ev3.status == "running"
    assert isinstance(ev4, SubAgentUpdated) and ev4.status == "completed"


@pytest.mark.asyncio
async def test_subagent_service_background_tracks_task_and_cleans_up():
    deps = _make_deps()
    bus: EventBus = EventBus()

    service = SubAgentService(
        deps=deps,
        event_bus=bus,
        agent_factory=lambda: _DummyAgent("ok", delay=0.01),
    )

    task_id, result = await service.run(prompt="P", description="D", background=True)
    assert result is None
    assert task_id in deps.sub_agent_tasks

    await asyncio.wait_for(deps.sub_agent_tasks[task_id], timeout=1)
    assert task_id not in deps.sub_agent_tasks
    assert deps.sub_agents[task_id].status in ("completed", "failed")




"""
MiniCC TUI 应用（多行输入 + @ 引用文件）

关键点：
- 输入使用 TextArea（ChatInput）：Enter 提交；Ctrl+J 换行
- 工具调用展示：消费 pydantic-ai stream events
- ask_user/todo/subagent：消费事件总线（core.events.EventBus）
"""

from __future__ import annotations

import os
import re
import subprocess
import traceback
from typing import Any

from textual import work
from textual.app import App, ComposeResult
from textual.binding import Binding
from textual.containers import Container, VerticalScroll
from textual.widgets import Footer, Header

from pydantic_ai import AgentRunResultEvent
from pydantic_ai.messages import (
    BuiltinToolCallEvent,
    BuiltinToolResultEvent,
    FunctionToolCallEvent,
    FunctionToolResultEvent,
    PartDeltaEvent,
    PartStartEvent,
    RetryPromptPart,
    TextPart,
    TextPartDelta,
    ToolReturnPart,
)

from minicc.core.config import load_config
from minicc.core.events import (
    AskUserRequested,
    SubAgentCreated,
    SubAgentUpdated,
    TodoUpdated,
    ToolCallFinished,
    ToolCallStarted,
)
from minicc.core.models import ToolResult, UserCancelledError
from minicc.core.runtime import MiniCCRuntime, build_runtime
from minicc.tui.ask_user_panel import AskUserPanel
from minicc.tui.chat_input import ChatInput
from minicc.tui.file_mention_panel import FileMentionPanel
from minicc.tui.widgets import BottomBar, MessagePanel, SubAgentLine, TodoDisplay, ToolCallLine


class MiniCCApp(App):
    TITLE = "MiniCC"
    CSS_PATH = "styles.tcss"

    BINDINGS = [
        Binding("ctrl+c", "quit", "退出", priority=True),
        Binding("ctrl+l", "clear", "清屏"),
        Binding("escape", "cancel", "取消"),
    ]

    def __init__(self, runtime: MiniCCRuntime | None = None):
        super().__init__()
        config = load_config()
        self.runtime = runtime or build_runtime(config=config, cwd=os.getcwd())
        self.messages: list[Any] = []
        self._is_processing = False
        self._git_branch = self._get_git_branch()

        self._tool_lines: dict[str, ToolCallLine] = {}
        self._subagent_lines: dict[str, SubAgentLine] = {}
        self._current_ask_panel: AskUserPanel | None = None
        self._streaming_assistant_panel: MessagePanel | None = None

        # @ 引用文件
        self._mention_active = False
        self._mention_at_pos: int | None = None  # 当前行内 @ 的索引
        self._mention_query = ""
        self._mention_items: list[str] = []
        self._mention_selected = 0

    def _get_git_branch(self) -> str | None:
        try:
            result = subprocess.run(
                ["git", "rev-parse", "--abbrev-ref", "HEAD"],
                capture_output=True,
                text=True,
                cwd=self.runtime.cwd,
                timeout=2,
            )
            if result.returncode == 0:
                return result.stdout.strip()
        except Exception:
            pass
        return None

    def compose(self) -> ComposeResult:
        yield Header(show_clock=True)
        yield VerticalScroll(id="chat_container")
        yield TodoDisplay(id="todo_display")
        yield Container(id="ask_user_container")
        yield Container(id="mention_container")
        yield ChatInput(
            id="input",
            placeholder="输入消息… Enter 发送，Ctrl+J 换行",
            soft_wrap=True,
            show_line_numbers=False,
        )
        yield BottomBar(
            model=f"{self.runtime.config.provider.value}:{self.runtime.config.model}",
            cwd=self.runtime.cwd,
            git_branch=self._git_branch,
            id="bottom_bar",
        )
        yield Footer(id="footer")

    def on_mount(self) -> None:
        input_widget = self.query_one("#input", ChatInput)
        input_widget.focus()
        input_widget.set_mention_key_handler(self._handle_mention_key)

        self.query_one("#todo_display", TodoDisplay).display = False
        self.query_one("#ask_user_container", Container).display = False
        self.query_one("#mention_container", Container).display = False
        self._show_welcome()
        self._wait_fs_ready()
        self._consume_events()

    @work(thread=True, group="startup")
    def _wait_fs_ready(self) -> None:
        try:
            self.runtime.fs.wait_ready(timeout=30.0)
        except Exception:
            pass

    def _show_welcome(self) -> None:
        self._append_message("**MiniCC** - 极简 AI 编程助手\n\n输入问题开始对话，Ctrl+C 退出", role="system")

    async def on_chat_input_submitted(self, event: ChatInput.Submitted) -> None:
        raw_input = event.value
        if not raw_input.strip():
            return

        if self._is_processing:
            self._append_message("⚠️ 请等待当前请求完成...", role="system")
            return

        input_widget = self.query_one("#input", ChatInput)
        self._hide_mention_panel()
        input_widget.text = ""
        input_widget.cursor_location = (0, 0)
        input_widget.call_after_refresh(input_widget.scroll_cursor_visible)

        user_input = raw_input
        self._append_message(user_input, role="user")

        self._streaming_assistant_panel = None
        self._process_message(user_input)

    def on_text_area_changed(self, event: ChatInput.Changed) -> None:
        if getattr(event, "text_area", None) is None or event.text_area.id != "input":
            return

        if self._current_ask_panel is not None:
            self._hide_mention_panel()
            return

        input_widget = self.query_one("#input", ChatInput)
        cursor_row, cursor_col = input_widget.cursor_location
        lines = input_widget.text.split("\n")
        current_line = lines[cursor_row] if 0 <= cursor_row < len(lines) else ""
        prefix = current_line[:cursor_col]

        at_info = _find_at_reference(prefix, len(prefix))
        if at_info is None:
            self._hide_mention_panel()
            return

        at_pos, query = at_info
        if query == "":
            self._show_mention_panel(at_pos, query, [])
            return

        items = self._search_files_for_mention(query)
        self._show_mention_panel(at_pos, query, items)

    @work(exclusive=True, group="chat")
    async def _process_message(self, user_input: str) -> None:
        self._is_processing = True
        try:
            streamed_text = ""
            async for event in self.runtime.agent.run_stream_events(
                user_input,
                deps=self.runtime.deps,
                message_history=self.messages,
            ):
                if isinstance(event, PartStartEvent) and isinstance(event.part, TextPart):
                    streamed_text += event.part.content
                    self._update_streaming_assistant(streamed_text)
                elif isinstance(event, PartDeltaEvent) and isinstance(event.delta, TextPartDelta):
                    streamed_text += event.delta.content_delta
                    self._update_streaming_assistant(streamed_text)
                elif isinstance(event, (FunctionToolCallEvent, BuiltinToolCallEvent)):
                    part = event.part
                    args = None
                    try:
                        args = part.args_as_dict()
                    except Exception:
                        try:
                            args = part.args if isinstance(part.args, dict) else None
                        except Exception:
                            args = None
                    self.runtime.event_bus.emit(
                        ToolCallStarted(tool_call_id=part.tool_call_id, tool_name=part.tool_name, args=args)
                    )
                elif isinstance(event, (FunctionToolResultEvent, BuiltinToolResultEvent)):
                    result_part = event.result
                    tool_name = getattr(result_part, "tool_name", "") or ""
                    ok, err = _tool_result_to_status(result_part)
                    self.runtime.event_bus.emit(
                        ToolCallFinished(
                            tool_call_id=result_part.tool_call_id,
                            tool_name=tool_name,
                            ok=ok,
                            content=getattr(result_part, "content", None),
                            error=err,
                        )
                    )
                elif isinstance(event, AgentRunResultEvent):
                    final_text = streamed_text or str(event.result.output)
                    if self._streaming_assistant_panel is not None:
                        self._streaming_assistant_panel.set_content(final_text)
                        self._scroll_chat_end()
                    else:
                        self._append_message(final_text, role="assistant")
                    self.messages = event.result.all_messages()
                    usage = event.result.usage()
                    if usage:
                        self._update_tokens(usage)

        except UserCancelledError:
            self._append_message("⚠️ 操作已取消", role="system")
        except Exception as e:
            if os.environ.get("MINICC_DEBUG"):
                tb = traceback.format_exc()
                self._append_message(f"❌ 错误: {e}\n\n```text\n{tb}\n```", role="system")
            else:
                self._append_message(f"❌ 错误: {e}", role="system")
        finally:
            self._is_processing = False
            self._scroll_chat_end()

    @work(group="events")
    async def _consume_events(self) -> None:
        async for ev in self.runtime.event_bus.iter():
            if isinstance(ev, ToolCallStarted):
                self._on_tool_started(ev)
            elif isinstance(ev, ToolCallFinished):
                self._on_tool_finished(ev)
            elif isinstance(ev, TodoUpdated):
                self._on_todo_updated(ev)
            elif isinstance(ev, AskUserRequested):
                self._on_ask_user_requested(ev)
            elif isinstance(ev, SubAgentCreated):
                self._on_subagent_created(ev)
            elif isinstance(ev, SubAgentUpdated):
                self._on_subagent_updated(ev)

    def _on_tool_started(self, ev: ToolCallStarted) -> None:
        line = ToolCallLine(ev.tool_name, ev.args, status="running")
        self._tool_lines[ev.tool_call_id] = line
        chat = self._chat_container()
        chat.mount(line)
        self._ensure_stream_panel_last()
        self._scroll_chat_end()

    def _on_tool_finished(self, ev: ToolCallFinished) -> None:
        line = self._tool_lines.get(ev.tool_call_id)
        if line is None:
            line = ToolCallLine(ev.tool_name, {}, status="running")
            self._tool_lines[ev.tool_call_id] = line
            self._chat_container().mount(line)
        line.update_status("completed" if ev.ok else "failed")
        self._ensure_stream_panel_last()
        self._scroll_chat_end()

    def _on_todo_updated(self, ev: TodoUpdated) -> None:
        todo_display = self.query_one("#todo_display", TodoDisplay)
        todo_display.update_todos(ev.todos)
        todo_display.display = len(ev.todos) > 0

    def _on_ask_user_requested(self, ev: AskUserRequested) -> None:
        container = self.query_one("#ask_user_container", Container)
        container.remove_children()
        panel = AskUserPanel(ev.request_id, ev.questions)
        self._current_ask_panel = panel
        container.mount(panel)
        container.display = True

        main_input = self.query_one("#input", ChatInput)
        main_input.disabled = True
        self.call_later(panel.focus)

    def _hide_ask_panel(self) -> None:
        try:
            container = self.query_one("#ask_user_container", Container)
            container.remove_children()
            container.display = False
        except Exception:
            pass

        self._current_ask_panel = None
        try:
            main_input = self.query_one("#input", ChatInput)
            main_input.disabled = False
            main_input.focus()
        except Exception:
            pass

        self._hide_mention_panel()

    def on_ask_user_panel_submitted(self, event: AskUserPanel.Submitted) -> None:
        self.runtime.deps.ask_user_service.resolve(event.request_id, submitted=True, answers=event.answers)
        self._hide_ask_panel()

    def on_ask_user_panel_cancelled(self, event: AskUserPanel.Cancelled) -> None:
        self.runtime.deps.ask_user_service.resolve(event.request_id, submitted=False, answers={})
        self._hide_ask_panel()

    def _on_subagent_created(self, ev: SubAgentCreated) -> None:
        line = SubAgentLine(task_id=ev.task_id, prompt=ev.description or ev.prompt, status="pending")
        self._subagent_lines[ev.task_id] = line
        self._chat_container().mount(line)
        self._ensure_stream_panel_last()
        self._scroll_chat_end()

    def _on_subagent_updated(self, ev: SubAgentUpdated) -> None:
        line = self._subagent_lines.get(ev.task_id)
        if line is None:
            line = SubAgentLine(task_id=ev.task_id, prompt=ev.task_id, status=ev.status)
            self._subagent_lines[ev.task_id] = line
            self._chat_container().mount(line)
        line.update_status(ev.status)
        self._ensure_stream_panel_last()
        self._scroll_chat_end()

    def on_todo_display_closed(self, message: TodoDisplay.Closed) -> None:
        todo_display = self.query_one("#todo_display", TodoDisplay)
        todo_display.update_todos([])
        todo_display.display = False
        self.runtime.deps.todos = []

    def action_clear(self) -> None:
        chat = self._chat_container()
        for child in list(chat.children):
            child.remove()
        self.messages = []
        self._tool_lines.clear()
        self._subagent_lines.clear()
        self._streaming_assistant_panel = None

        try:
            bottom_bar = self.query_one(BottomBar)
            bottom_bar.update_info(input_tokens=0, output_tokens=0)
        except Exception:
            pass

        todo_display = self.query_one("#todo_display", TodoDisplay)
        todo_display.update_todos([])
        todo_display.display = False
        self.runtime.deps.todos = []
        self._show_welcome()

    def action_quit(self) -> None:
        self.runtime.close()
        self.exit()

    def action_cancel(self) -> None:
        if self._is_processing:
            self._append_message("⚠️ 正在取消...", role="system")

    def _chat_container(self) -> VerticalScroll:
        return self.query_one("#chat_container", VerticalScroll)

    def _append_message(self, content: str, role: str = "assistant") -> MessagePanel:
        panel = MessagePanel(content, role=role)
        chat = self._chat_container()
        chat.mount(panel)
        self._scroll_chat_end()
        return panel

    def _scroll_chat_end(self) -> None:
        chat = self._chat_container()
        chat.call_after_refresh(chat.scroll_end, animate=False)

    def _update_streaming_assistant(self, content: str) -> None:
        if self._streaming_assistant_panel is None:
            self._streaming_assistant_panel = self._append_message("", role="assistant")
        self._streaming_assistant_panel.set_content(content)
        self._scroll_chat_end()

    def _ensure_stream_panel_last(self) -> None:
        if self._streaming_assistant_panel is None:
            return
        chat = self._chat_container()
        try:
            self._streaming_assistant_panel.remove()
        except Exception:
            return
        chat.mount(self._streaming_assistant_panel)

    def _update_tokens(self, usage: Any) -> None:
        try:
            bottom_bar = self.query_one(BottomBar)
            input_tokens = getattr(usage, "request_tokens", 0) or getattr(usage, "input_tokens", 0)
            output_tokens = getattr(usage, "response_tokens", 0) or getattr(usage, "output_tokens", 0)
            bottom_bar.add_tokens(input_tokens, output_tokens)
        except Exception:
            pass

    # ---------------- @ 引用文件 ----------------

    def _show_mention_panel(self, at_pos: int, query: str, items: list[str]) -> None:
        self._mention_active = True
        self._mention_at_pos = at_pos
        self._mention_query = query
        self._mention_items = items
        self._mention_selected = 0
        self._refresh_mention_panel()
        self.call_later(self.query_one("#input", ChatInput).focus)

    def _refresh_mention_panel(self) -> None:
        container = self.query_one("#mention_container", Container)
        container.remove_children()
        panel = FileMentionPanel(self._mention_query, self._mention_items, self._mention_selected)
        container.mount(panel)
        container.display = True

    def _hide_mention_panel(self) -> None:
        if not self._mention_active:
            return
        self._mention_active = False
        self._mention_at_pos = None
        self._mention_query = ""
        self._mention_items = []
        self._mention_selected = 0
        try:
            container = self.query_one("#mention_container", Container)
            container.remove_children()
            container.display = False
        except Exception:
            pass

    def _handle_mention_key(self, key: str) -> bool:
        if not self._mention_active:
            return False
        if key == "escape":
            self._hide_mention_panel()
            return True
        if key in ("up", "down"):
            if not self._mention_items:
                return True
            if key == "up":
                self._mention_selected = max(0, self._mention_selected - 1)
            else:
                self._mention_selected = min(len(self._mention_items) - 1, self._mention_selected + 1)
            self._refresh_mention_panel()
            return True
        if key in ("tab", "enter"):
            if self._mention_items:
                self._accept_mention()
                return True
        return False

    def _accept_mention(self) -> None:
        if not self._mention_items or self._mention_at_pos is None:
            self._hide_mention_panel()
            return

        selected = self._mention_items[self._mention_selected]
        input_widget = self.query_one("#input", ChatInput)
        cursor_row, cursor_col = input_widget.cursor_location
        lines = (input_widget.text.split("\n") or [""])
        if cursor_row >= len(lines):
            lines += [""] * (cursor_row - len(lines) + 1)

        line = lines[cursor_row]
        at_pos = self._mention_at_pos
        before = line[: at_pos + 1]
        after = line[cursor_col:]
        insert = selected + " "
        lines[cursor_row] = before + insert + after
        input_widget.text = "\n".join(lines)
        input_widget.cursor_location = (cursor_row, len(before) + len(insert))
        input_widget.call_after_refresh(input_widget.scroll_cursor_visible)

        self._hide_mention_panel()
        self.call_later(input_widget.focus)

    def _search_files_for_mention(self, query: str) -> list[str]:
        fs = getattr(self.runtime, "fs", None)
        ignored = {".git", ".venv", "dist", "__pycache__", ".pytest_cache"}

        def is_ignored(p: str) -> bool:
            parts = p.split("/")
            return any(x in ignored for x in parts)

        patterns: list[str]
        if "/" in query or query.startswith("."):
            patterns = [f"{query}*"]
        else:
            patterns = [f"**/*{query}*"]

        results: list[str] = []
        seen: set[str] = set()
        for pat in patterns:
            try:
                matches = fs.glob(pat) if fs is not None else []
            except Exception:
                matches = []
            for m in matches:
                if m in seen:
                    continue
                if is_ignored(m):
                    continue
                try:
                    if not (self.runtime.cwd and os.path.isfile(os.path.join(self.runtime.cwd, m))):
                        continue
                except Exception:
                    continue
                seen.add(m)
                results.append(m)
                if len(results) >= 100:
                    return results
        return results


_AT_PATTERN = re.compile(r"(^|[\\s\\(\\[\\{\\\"'])@([^\\s@]*)$")


def _find_at_reference(text: str, cursor_pos: int) -> tuple[int, str] | None:
    prefix = text[:cursor_pos]
    m = _AT_PATTERN.search(prefix)
    if not m:
        return None
    at_pos = m.start(0) + len(m.group(1))
    query = m.group(2)
    return at_pos, query


def _tool_result_to_status(result_part: ToolReturnPart | RetryPromptPart) -> tuple[bool, str | None]:
    if isinstance(result_part, RetryPromptPart):
        return False, str(result_part.content)
    content = result_part.content
    if isinstance(content, ToolResult):
        return bool(content.success), content.error
    if hasattr(content, "success") and hasattr(content, "error"):
        try:
            ok = bool(getattr(content, "success"))
            err = getattr(content, "error", None)
            return ok, err
        except Exception:
            pass
    return True, None


def main() -> None:
    MiniCCApp().run()



"""
MiniCC TUI 组件

说明：本版本将“工具调用状态更新”作为一等能力，ToolCallLine/SubAgentLine 支持状态刷新。
"""

from __future__ import annotations

from rich.markdown import Markdown
from rich.panel import Panel
from rich.text import Text
from textual.message import Message
from textual.widgets import Static

from minicc.core.models import DiffLine, TodoItem


class MessagePanel(Static):
    def __init__(self, content: str, role: str = "user", **kwargs):
        self.role = role
        self._content = content
        super().__init__(content, markup=False, **kwargs)

    def set_content(self, content: str) -> None:
        self._content = content
        self.update(content)

    def render(self) -> Panel:
        role_style = {
            "user": ("blue", "You"),
            "assistant": ("green", "Assistant"),
            "system": ("magenta", "System"),
        }
        color, title = role_style.get(self.role, ("white", self.role.title()))
        markdown = Markdown(self._content or "", code_theme="monokai", justify="left")
        return Panel(markdown, title=title, border_style=color, expand=True)


class ToolCallLine(Static):
    def __init__(self, tool_name: str, args: dict | None, status: str = "running", **kwargs):
        self.tool_name = tool_name
        self.args = args or {}
        self.status = status
        super().__init__(**kwargs)

    def update_status(self, status: str) -> None:
        self.status = status
        self.refresh()

    def render(self) -> Text:
        text = Text()
        text.append("  🔧 ", style="yellow")
        text.append(self.tool_name, style="bold yellow")

        summary = self._get_summary()
        if summary:
            text.append(f" {summary}", style="dim")

        icon = {
            "pending": " ⏳",
            "running": " 🔄",
            "completed": " ✅",
            "failed": " ❌",
        }.get(self.status, " ❓")
        style = {"completed": "green", "failed": "red", "running": "yellow", "pending": "dim"}.get(
            self.status, "dim"
        )
        text.append(icon, style=style)
        return text

    def _get_summary(self) -> str:
        key_params = ["path", "file_path", "pattern", "command", "query", "prompt"]
        for key in key_params:
            if key in self.args:
                value = str(self.args[key])
                if len(value) > 40:
                    value = value[:40] + "..."
                return f"({value})"
        return ""


class SubAgentLine(Static):
    def __init__(self, task_id: str, prompt: str, status: str, **kwargs):
        self.task_id = task_id
        self.prompt = prompt
        self.status = status
        super().__init__(**kwargs)

    def update_status(self, status: str) -> None:
        self.status = status
        self.refresh()

    def render(self) -> Text:
        text = Text()
        text.append("  🤖 ", style="magenta")
        prompt_short = self.prompt[:50] + "..." if len(self.prompt) > 50 else self.prompt
        text.append(prompt_short, style="bold magenta")
        icon = {
            "pending": " ⏳",
            "running": " 🔄",
            "completed": " ✅",
            "failed": " ❌",
        }.get(self.status, " ❓")
        text.append(icon)
        return text


class DiffView(Static):
    def __init__(self, diff_lines: list[DiffLine], filename: str = "", **kwargs):
        self.diff_lines = diff_lines
        self.filename = filename
        super().__init__(**kwargs)

    def render(self) -> Panel:
        text = Text()
        for line in self.diff_lines:
            if line.type == "add":
                text.append(f"+ {line.content}\n", style="green")
            elif line.type == "remove":
                text.append(f"- {line.content}\n", style="red")
            else:
                text.append(f"  {line.content}\n", style="dim")
        title = f"Diff: {self.filename}" if self.filename else "Diff"
        return Panel(text, title=title, border_style="cyan", expand=True)


class BottomBar(Static):
    def __init__(
        self,
        model: str = "",
        cwd: str = "",
        git_branch: str | None = None,
        input_tokens: int = 0,
        output_tokens: int = 0,
        **kwargs,
    ):
        self.model = model
        self.cwd = cwd
        self.git_branch = git_branch
        self.input_tokens = input_tokens
        self.output_tokens = output_tokens
        super().__init__(**kwargs)

    def update_info(
        self,
        model: str | None = None,
        cwd: str | None = None,
        git_branch: str | None = None,
        input_tokens: int | None = None,
        output_tokens: int | None = None,
    ) -> None:
        if model is not None:
            self.model = model
        if cwd is not None:
            self.cwd = cwd
        if git_branch is not None:
            self.git_branch = git_branch
        if input_tokens is not None:
            self.input_tokens = input_tokens
        if output_tokens is not None:
            self.output_tokens = output_tokens
        self.refresh()

    def add_tokens(self, input_delta: int = 0, output_delta: int = 0) -> None:
        self.input_tokens += input_delta
        self.output_tokens += output_delta
        self.refresh()

    def render(self) -> Text:
        text = Text()
        text.append(" 📦 ", style="dim")
        text.append(self.model or "N/A", style="cyan")
        text.append("  │  ", style="dim")

        text.append("📁 ", style="dim")
        cwd_short = self.cwd
        if len(cwd_short) > 25:
            cwd_short = "..." + cwd_short[-22:]
        text.append(cwd_short, style="green")
        text.append("  │  ", style="dim")

        text.append("🌿 ", style="dim")
        text.append(self.git_branch or "N/A", style="magenta" if self.git_branch else "dim")
        text.append("  │  ", style="dim")

        # 说明：部分终端/字体对 emoji（如 ⬆️/⬇️）支持不佳，容易显示为方块或宽度异常；
        # 因此使用更通用的箭头字符。
        text.append("↑", style="dim")
        text.append(f"{self.input_tokens}", style="yellow")
        text.append(" ↓", style="dim")
        text.append(f"{self.output_tokens}", style="yellow")
        return text


class TodoDisplay(Static):
    class Closed(Message):
        pass

    def __init__(self, todos: list[TodoItem] | None = None, **kwargs):
        self.todos: list[TodoItem] = todos or []
        super().__init__(**kwargs)

    def update_todos(self, todos: list[TodoItem]) -> None:
        self.todos = todos
        self.refresh()

    def is_all_completed(self) -> bool:
        if not self.todos:
            return False
        return all(t.status == "completed" for t in self.todos)

    async def on_click(self, event) -> None:
        if self.is_all_completed():
            if event.x >= self.size.width - 6:
                self.post_message(self.Closed())

    def render(self) -> Panel:
        if not self.todos:
            return Panel(Text("暂无任务", style="dim"), title="📋 任务", border_style="dim")

        text = Text()
        pending = [t for t in self.todos if t.status in ("pending", "in_progress")]
        completed = [t for t in self.todos if t.status == "completed"]
        total = len(self.todos)
        done = len(completed)

        if pending:
            for todo in pending:
                if todo.status == "in_progress":
                    text.append("🔄 ", style="yellow")
                    text.append(f"{todo.active_form}\n", style="yellow bold")
                else:
                    text.append("⏳ ", style="dim")
                    text.append(f"{todo.content}\n", style="dim")

        if completed:
            if pending:
                text.append("─" * 20 + "\n", style="dim")
            text.append(f"✅ 已完成 {done} 项", style="green dim")
            recent = completed[-3:] if len(completed) > 3 else completed
            for todo in recent:
                text.append(f"\n   ✓ {todo.content}", style="green dim")
            if len(completed) > 3:
                text.append(f"\n   ... 及其他 {len(completed) - 3} 项", style="dim")

        all_done = done == total and total > 0
        title = "📋 任务 ✓ 全部完成 [×]" if all_done else f"📋 任务 [{done}/{total}]"
        border = "green" if all_done else "cyan"
        return Panel(text, title=title, title_align="left", border_style=border)



from __future__ import annotations

import pytest

from minicc.core.events import EventBus
from minicc.core.models import Config, MiniCCDeps, Provider


@pytest.fixture
def event_bus() -> EventBus:
    return EventBus()


@pytest.fixture
def config() -> Config:
    return Config(provider=Provider.ANTHROPIC, model="test-model", api_key="test-key")


@pytest.fixture
def deps(config: Config) -> MiniCCDeps:
    return MiniCCDeps(config=config, cwd="/tmp/minicc-test", fs=None)




from __future__ import annotations

import json
import re
from collections import Counter
from json import JSONDecodeError

from pydantic_ai import RunContext
from pydantic import TypeAdapter, ValidationError

from minicc.core.models import MiniCCDeps, Question, QuestionOption, ToolResult


def _normalize_ask_user_questions(questions: list[Question]) -> list[Question]:
    """
    归一化 ask_user 的 questions 输入。

    - 清理 header 空白并保证唯一（避免答案 key 覆盖）
    - question 为空时回退为 header
    - 选项 label 做 strip（展示更稳定）
    """

    normalized: list[Question] = []
    provisional_headers: list[str] = []

    for i, q in enumerate(questions):
        question_text = (q.question or "").strip()
        header = (q.header or "").strip()
        header = re.sub(r"\s+", " ", header).strip()
        provisional_headers.append(header)

        options = [
            QuestionOption(label=(opt.label or "").strip(), description=opt.description or "")
            for opt in q.options or []
        ]

        normalized.append(
            Question(
                question=question_text or header,
                header=header,
                options=options,
                multi_select=bool(q.multi_select),
            )
        )

    # 保证 header 非空且唯一，避免 answers_out key 覆盖。
    counts = Counter(provisional_headers)
    used: dict[str, int] = {}
    for idx, question in enumerate(normalized):
        header = question.header
        if counts[header] <= 1:
            continue
        used[header] = used.get(header, 0) + 1
        normalized[idx] = question.model_copy(update={"header": f"{header}#{used[header]}"})

    return normalized


def _validate_and_normalize_ask_user_questions(payload: object) -> list[Question]:
    """
    将 LLM 传入的 ask_user 参数转为 list[Question]，并做强校验，避免 UI 异常与难懂的 AttributeError。
    """

    data: object = payload
    if isinstance(data, str):
        try:
            data = json.loads(data)
        except JSONDecodeError as e:
            raise ValueError(
                "ask_user 参数错误：`questions` 必须是 JSON 数组（list），每项包含 `header`、`question`、`options`。"
            ) from e

    if isinstance(data, dict) and "questions" in data:
        data = data["questions"]

    try:
        questions = TypeAdapter(list[Question]).validate_python(data)
    except ValidationError as e:
        first = e.errors()[0] if e.errors() else {"loc": (), "msg": str(e)}
        loc = ".".join(str(x) for x in first.get("loc", ())) or "questions"
        msg = first.get("msg", "参数不合法")
        raise ValueError(f"ask_user 参数错误：{loc} - {msg}") from e

    for i, q in enumerate(questions):
        header = (q.header or "").strip()
        if not header:
            raise ValueError(f"ask_user 参数错误：第 {i + 1} 题缺少 `header`（必须提供且唯一）")

        if not q.options or all(not (opt.label or "").strip() for opt in q.options):
            raise ValueError(f"ask_user 参数错误：第 {i + 1} 题缺少 `options`（至少 1 个，且每个需有 label）")

    return _normalize_ask_user_questions(questions)


async def ask_user(ctx: RunContext[MiniCCDeps], questions: list[Question]) -> ToolResult:
    """
    向用户提出 1~N 个选择题/多选题，并等待用户在 TUI 面板中提交或取消。

    重要：每个问题必须提供 `header` 与 `options`：
    - `header`：简短且唯一，用作答案 key（避免重复导致覆盖）
    - `question`：给用户看的问题文本
    - `options`：至少 1 个选项（推荐 2~6 个）

    UI 会额外提供“其他（自定义输入）”一项，允许用户自由输入。
    """
    service = ctx.deps.ask_user_service
    if service is None:
        return ToolResult(success=False, output="", error="ask_user_service 未初始化")

    try:
        parsed_questions = _validate_and_normalize_ask_user_questions(questions)
    except ValueError as e:
        return ToolResult(
            success=False,
            output="",
            error=(
                f"{e}\n示例："
                '\n[{"header":"语言","question":"你主要用什么编程语言？","options":[{"label":"Python"},{"label":"TypeScript"}]}]'
            ),
        )

    result = await service.ask(parsed_questions)
    return ToolResult(success=True, output=json.dumps(result.answers, ensure_ascii=False))



from __future__ import annotations

import asyncio
import os
import re
import signal
from uuid import uuid4

from pydantic_ai import RunContext

from minicc.core.models import BackgroundShell, MiniCCDeps, ToolResult
from minicc.tools.common import DEFAULT_BASH_TIMEOUT_MS, MAX_OUTPUT_CHARS


async def bash(
    ctx: RunContext[MiniCCDeps],
    command: str,
    timeout: int = DEFAULT_BASH_TIMEOUT_MS,
    description: str | None = None,
    run_in_background: bool = False,
) -> ToolResult:
    timeout = min(max(timeout, 1000), 600000)
    timeout_sec = timeout / 1000

    if run_in_background:
        return await _bash_background(ctx, command, description or command[:30])

    try:
        process = await asyncio.create_subprocess_shell(
            command,
            stdout=asyncio.subprocess.PIPE,
            stderr=asyncio.subprocess.PIPE,
            cwd=ctx.deps.cwd,
            preexec_fn=os.setsid if os.name == "posix" else None,
        )

        try:
            stdout, stderr = await asyncio.wait_for(process.communicate(), timeout=timeout_sec)
        except asyncio.TimeoutError:
            _kill_process_tree(process)
            return ToolResult(success=False, output="", error=f"命令执行超时（{timeout_sec:.1f}秒）")

        stdout_str = (stdout or b"").decode("utf-8", errors="replace")
        stderr_str = (stderr or b"").decode("utf-8", errors="replace")
        output = stdout_str
        if stderr_str:
            output = f"{output}\n[stderr]\n{stderr_str}" if output else stderr_str

        if len(output) > MAX_OUTPUT_CHARS:
            output = output[:MAX_OUTPUT_CHARS] + "\n... 输出已截断"

        success = process.returncode == 0
        error = None if success else f"退出码: {process.returncode}"
        return ToolResult(success=success, output=output, error=error)

    except Exception as e:
        return ToolResult(success=False, output="", error=str(e))


async def bash_output(
    ctx: RunContext[MiniCCDeps],
    bash_id: str,
    filter_pattern: str | None = None,
) -> ToolResult:
    shell_data = ctx.deps.background_shells.get(bash_id)
    if not shell_data:
        return ToolResult(success=False, output="", error=f"未找到后台任务: {bash_id}")

    process, shell_info = shell_data
    if process.returncode is not None:
        shell_info.is_running = False

    output = shell_info.output_buffer
    if filter_pattern and output:
        try:
            regex = re.compile(filter_pattern)
            output = "\n".join(line for line in output.split("\n") if regex.search(line))
        except re.error:
            pass

    status = "运行中" if shell_info.is_running else "已完成"
    return ToolResult(success=True, output=f"[{status}]\n{output}")


async def kill_shell(ctx: RunContext[MiniCCDeps], shell_id: str) -> ToolResult:
    shell_data = ctx.deps.background_shells.get(shell_id)
    if not shell_data:
        return ToolResult(success=False, output="", error=f"未找到后台任务: {shell_id}")

    process, shell_info = shell_data
    if shell_info.is_running:
        try:
            _kill_process_tree(process)
            shell_info.is_running = False
        except Exception as e:
            return ToolResult(success=False, output="", error=str(e))

    del ctx.deps.background_shells[shell_id]
    return ToolResult(success=True, output=f"已终止后台任务: {shell_id}")


def _kill_process_tree(process: asyncio.subprocess.Process) -> None:
    if os.name == "posix" and process.pid:
        try:
            os.killpg(process.pid, signal.SIGKILL)
        except ProcessLookupError:
            pass
    else:
        process.kill()


async def _bash_background(ctx: RunContext[MiniCCDeps], command: str, description: str) -> ToolResult:
    shell_id = uuid4().hex[:8]

    process = await asyncio.create_subprocess_shell(
        command,
        stdout=asyncio.subprocess.PIPE,
        stderr=asyncio.subprocess.STDOUT,
        cwd=ctx.deps.cwd,
        preexec_fn=os.setsid if os.name == "posix" else None,
    )

    shell_info = BackgroundShell(shell_id=shell_id, command=command, description=description, is_running=True)
    ctx.deps.background_shells[shell_id] = (process, shell_info)

    asyncio.create_task(_collect_shell_output(process, shell_info))
    return ToolResult(success=True, output=f"已在后台启动命令 [ID: {shell_id}]: {description}")


async def _collect_shell_output(process: asyncio.subprocess.Process, shell_info: BackgroundShell) -> None:
    try:
        while True:
            if process.stdout is None:
                break
            line = await process.stdout.readline()
            if not line:
                break
            shell_info.output_buffer += line.decode("utf-8", errors="replace")
    except Exception:
        pass
    finally:
        shell_info.is_running = False




"""
MiniCC MCP（Model Context Protocol）预加载

目标：
- 程序启动时就加载 MCP servers，并将 toolsets 传给 Agent（不再运行中动态/懒加载）。
- 缺少可选依赖时，默认降级为空 toolsets（可通过环境变量开启严格模式）。
"""

from __future__ import annotations

import os
import warnings
from pathlib import Path
from typing import Any

from pydantic_ai.toolsets import AbstractToolset

from .config import find_mcp_config

_CACHE: dict[str, list[AbstractToolset[Any]]] = {}


def load_mcp_toolsets(cwd: str | Path | None) -> list[AbstractToolset[Any]]:
    base = cwd if cwd is not None else os.getcwd()
    config_path = find_mcp_config(base)
    if not config_path:
        return []

    cache_key = str(Path(config_path).resolve())
    cached = _CACHE.get(cache_key)
    if cached is not None:
        return cached

    strict = os.environ.get("MINICC_MCP_STRICT") == "1"

    try:
        from pydantic_ai.mcp import load_mcp_servers
    except Exception as e:
        msg = f"加载 MCP 失败：未安装 MCP 依赖或导入异常：{e}"
        if strict:
            raise RuntimeError(msg) from e
        warnings.warn(msg)
        _CACHE[cache_key] = []
        return []

    try:
        servers = load_mcp_servers(Path(config_path))
        toolsets = list(servers or [])
    except Exception as e:
        msg = f"加载 MCP 失败：配置文件 {config_path} 解析/展开环境变量异常：{e}"
        if strict:
            raise RuntimeError(msg) from e
        warnings.warn(msg)
        toolsets = []

    _CACHE[cache_key] = toolsets
    return toolsets




"""
MiniCC Agent 创建与运行
"""

from __future__ import annotations

from pathlib import Path
from typing import Any, Callable

from pydantic_ai import Agent
from pydantic_ai.models.anthropic import AnthropicModel
from pydantic_ai.models.openai import OpenAIModel
from pydantic_ai.providers.anthropic import AnthropicProvider
from pydantic_ai.providers.openai import OpenAIProvider
from pydantic_ai.toolsets import AbstractToolset

from .config import get_api_key, load_agents_prompt
from .mcp import load_mcp_toolsets
from .models import Config, MiniCCDeps, Provider


def create_model(config: Config) -> AnthropicModel | OpenAIModel | str:
    api_key = get_api_key(config.provider)

    if config.base_url or config.api_key:
        if config.provider == Provider.ANTHROPIC:
            provider = AnthropicProvider(api_key=api_key, base_url=config.base_url)
            return AnthropicModel(config.model, provider=provider)
        provider = OpenAIProvider(api_key=api_key, base_url=config.base_url)
        return OpenAIModel(config.model, provider=provider)

    if config.provider == Provider.ANTHROPIC:
        return f"anthropic:{config.model}"
    return f"openai:{config.model}"


def _build_model_settings(config: Config) -> dict[str, Any] | None:
    if config.provider != Provider.ANTHROPIC:
        return None

    cache = config.prompt_cache
    settings: dict[str, Any] = {}
    if cache.instructions:
        settings["anthropic_cache_instructions"] = cache.instructions
    if cache.tool_definitions:
        settings["anthropic_cache_tool_definitions"] = cache.tool_definitions
    if cache.messages:
        settings["anthropic_cache_messages"] = cache.messages
    return settings or None


def create_agent(
    config: Config,
    *,
    cwd: str | Path | None = None,
    toolsets: list[AbstractToolset[Any]] | None = None,
    register_tools: Callable[[Agent[MiniCCDeps, str]], None] | None = None,
) -> Agent[MiniCCDeps, str]:
    """
    创建并配置主 Agent

    说明：
    - MCP toolsets 默认在此处加载，但 TUI 运行时会在启动阶段预加载并传入（避免重复加载）。
    - 工具注册通过 register_tools 回调注入（便于拆分 tools 模块）。
    """

    model = create_model(config)
    system_prompt = load_agents_prompt()
    model_settings = _build_model_settings(config)

    if toolsets is None:
        toolsets = load_mcp_toolsets(cwd)

    agent: Agent[MiniCCDeps, str] = Agent(
        model=model,
        deps_type=MiniCCDeps,
        system_prompt=system_prompt,
        model_settings=model_settings,
        toolsets=toolsets,
    )

    if register_tools is not None:
        register_tools(agent)

    return agent


async def run_agent(
    agent: Agent[MiniCCDeps, str],
    prompt: str,
    deps: MiniCCDeps,
    message_history: list | None = None,
) -> tuple[str, list]:
    result = await agent.run(prompt, deps=deps, message_history=message_history or [])
    return result.output, result.all_messages()



from __future__ import annotations

import builtins
import sys
import types
import warnings

import pytest

import minicc.core.mcp as mcp


def test_load_mcp_toolsets_no_config_returns_empty(monkeypatch):
    monkeypatch.setattr(mcp, "find_mcp_config", lambda _: None)
    monkeypatch.setattr(mcp, "_CACHE", {})
    assert mcp.load_mcp_toolsets("/tmp/x") == []


def test_load_mcp_toolsets_caches_by_config_path(monkeypatch):
    monkeypatch.setattr(mcp, "find_mcp_config", lambda _: "/tmp/minicc-mcp.json")
    monkeypatch.setattr(mcp, "_CACHE", {})

    calls = {"n": 0}
    mod = types.ModuleType("pydantic_ai.mcp")

    def load_mcp_servers(_):
        calls["n"] += 1
        return ["toolset-a"]

    mod.load_mcp_servers = load_mcp_servers
    monkeypatch.setitem(sys.modules, "pydantic_ai.mcp", mod)

    a = mcp.load_mcp_toolsets("/tmp/x")
    b = mcp.load_mcp_toolsets("/tmp/x")
    assert a == ["toolset-a"]
    assert b == ["toolset-a"]
    assert calls["n"] == 1


def test_load_mcp_toolsets_import_error_non_strict_warns_and_returns_empty(monkeypatch):
    monkeypatch.setattr(mcp, "find_mcp_config", lambda _: "/tmp/minicc-mcp.json")
    monkeypatch.setattr(mcp, "_CACHE", {})
    monkeypatch.delenv("MINICC_MCP_STRICT", raising=False)
    monkeypatch.setitem(sys.modules, "pydantic_ai.mcp", None)

    orig_import = builtins.__import__

    def fake_import(name, *args, **kwargs):
        if name == "pydantic_ai.mcp":
            raise ImportError("no mcp")
        return orig_import(name, *args, **kwargs)

    monkeypatch.setattr(builtins, "__import__", fake_import)

    with warnings.catch_warnings(record=True) as rec:
        warnings.simplefilter("always")
        out = mcp.load_mcp_toolsets("/tmp/x")
        assert out == []
        assert any("加载 MCP 失败" in str(w.message) for w in rec)


def test_load_mcp_toolsets_import_error_strict_raises(monkeypatch):
    monkeypatch.setattr(mcp, "find_mcp_config", lambda _: "/tmp/minicc-mcp.json")
    monkeypatch.setattr(mcp, "_CACHE", {})
    monkeypatch.setenv("MINICC_MCP_STRICT", "1")
    monkeypatch.setitem(sys.modules, "pydantic_ai.mcp", None)

    orig_import = builtins.__import__

    def fake_import(name, *args, **kwargs):
        if name == "pydantic_ai.mcp":
            raise ImportError("no mcp")
        return orig_import(name, *args, **kwargs)

    monkeypatch.setattr(builtins, "__import__", fake_import)

    with pytest.raises(RuntimeError):
        mcp.load_mcp_toolsets("/tmp/x")



from __future__ import annotations

import asyncio

import pytest

from minicc.core.events import EventBus


@pytest.mark.asyncio
async def test_event_bus_next_order():
    bus: EventBus[int] = EventBus()
    bus.emit(1)
    bus.emit(2)
    assert await bus.next() == 1
    assert await bus.next() == 2


@pytest.mark.asyncio
async def test_event_bus_iter_yields_values():
    bus: EventBus[str] = EventBus()

    async def consume_two() -> list[str]:
        out: list[str] = []
        async for ev in bus.iter():
            out.append(ev)
            if len(out) == 2:
                return out
        return out

    task = asyncio.create_task(consume_two())
    bus.emit("a")
    bus.emit("b")
    assert await asyncio.wait_for(task, timeout=1) == ["a", "b"]




"""
聊天输入框（支持 Enter 提交、Ctrl+J 换行）

Textual 的 Input 组件不适合多行输入；这里基于 TextArea 实现一个轻量输入控件：
- Enter：提交（不会插入换行）
- Ctrl+J：插入换行
- 其余按键：沿用 TextArea 的编辑行为

另外支持可选的“@ 引用文件”键盘控制委托（up/down/enter/tab/escape）。
"""

from __future__ import annotations

from collections.abc import Callable

from textual import events
from textual.message import Message
from textual.widgets import TextArea


class ChatInput(TextArea):
    class Submitted(Message):
        def __init__(self, value: str):
            self.value = value
            super().__init__()

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self._mention_key_handler: Callable[[str], bool] | None = None

    def set_mention_key_handler(self, handler: Callable[[str], bool] | None) -> None:
        self._mention_key_handler = handler

    async def _on_key(self, event: events.Key) -> None:
        # 先让 @ 引用面板拦截（如果已开启）
        if self._mention_key_handler is not None:
            try:
                if self._mention_key_handler(event.key):
                    event.stop()
                    event.prevent_default()
                    return
            except Exception:
                pass

        if self.read_only:
            return

        if event.key == "ctrl+j":
            event.stop()
            event.prevent_default()
            start, end = self.selection
            self._replace_via_keyboard("\n", start, end)
            self.scroll_cursor_visible()
            return

        if event.key == "enter":
            event.stop()
            event.prevent_default()
            self.post_message(self.Submitted(self.text))
            return

        # 其他按键走 TextArea 默认逻辑（含可打印字符插入、光标移动等）
        await super()._on_key(event)



"""
MiniCC Ask User Panel 组件

提供 ask_user 工具的可交互问答面板。
"""

from __future__ import annotations

from rich.panel import Panel
from rich.text import Text
from textual.message import Message
from textual.widgets import Static

from minicc.core.models import Question


class AskUserPanel(Static, can_focus=True):
    BINDINGS = [
        ("left", "prev_question", "上一个问题"),
        ("right", "next_question", "下一个问题"),
        ("up", "prev_option", "上一个选项"),
        ("down", "next_option", "下一个选项"),
        ("enter", "select_option", "选择"),
        ("escape", "cancel", "取消"),
    ]

    class Submitted(Message):
        def __init__(self, request_id: str, answers: dict[str, str | list[str]]):
            self.request_id = request_id
            self.answers = answers
            super().__init__()

    class Cancelled(Message):
        def __init__(self, request_id: str):
            self.request_id = request_id
            super().__init__()

    def __init__(self, request_id: str, questions: list[Question], **kwargs):
        self.request_id = request_id
        self.questions = questions
        self.current_question = 0
        self.current_option = 0
        self.typing_mode = False
        self.typing_buffer = ""
        self.answers: dict[int, int | set[int]] = {}
        self.custom_inputs: dict[int, str] = {}

        for i, q in enumerate(self.questions):
            self.answers[i] = set() if q.multi_select else -1
        super().__init__(**kwargs)

    def render(self) -> Panel:
        text = Text()
        text.append("← ", style="dim")
        for i, q in enumerate(self.questions):
            if i == self.current_question:
                text.append(f" □ {q.header} ", style="bold reverse magenta")
            else:
                text.append(f" □ {q.header} ", style="green" if self._is_answered(i) else "dim")
        text.append(" →\n\n", style="dim")

        q = self.questions[self.current_question]
        multi_hint = "（可多选）" if q.multi_select else ""
        text.append(f"{q.question} {multi_hint}\n\n", style="bold cyan")

        for j, opt in enumerate(q.options):
            is_selected = self.current_option == j and not self.typing_mode
            is_checked = self._is_option_checked(self.current_question, j)

            text.append("❯ " if is_selected else "  ", style="bold yellow" if is_selected else "")
            text.append(f"{j + 1}. ", style="bold" if is_selected else "dim")

            if is_checked:
                text.append(opt.label, style="bold green")
                if q.multi_select:
                    text.append(" ✓", style="green")
            else:
                text.append(opt.label, style="bold" if is_selected else "")
            text.append("\n")
            if opt.description:
                text.append(f"    {opt.description}\n", style="dim italic")

        custom_idx = len(q.options)
        is_selected = self.current_option == custom_idx or self.typing_mode
        is_custom_active = self._is_custom_selected(self.current_question)
        text.append("❯ " if is_selected else "  ", style="bold yellow" if is_selected else "")
        text.append(f"{custom_idx + 1}. ", style="bold" if is_selected else "dim")

        custom_text = self.custom_inputs.get(self.current_question, "")
        if self.typing_mode:
            text.append("其他（自定义）：", style="dim")
            text.append(self.typing_buffer, style="bold cyan")
            text.append("█", style="bold cyan blink")
        elif custom_text:
            text.append(
                f"其他（自定义）：{custom_text}",
                style="bold green" if is_custom_active else "green",
            )
            if q.multi_select and is_custom_active:
                text.append(" ✓", style="green")
        else:
            text.append("其他（自定义输入）", style="bold" if is_selected else "dim italic")

        text.append("\n\n")

        all_answered = all(self._is_answered(i) for i in range(len(self.questions)))
        if self.typing_mode:
            text.append("输入内容，Enter 确认，Esc 取消", style="dim")
        elif all_answered:
            text.append("✔ 全部已选择，按 ", style="green")
            text.append("S", style="bold green reverse")
            text.append(" 提交 · Esc 取消", style="green")
        else:
            text.append("Enter 选择 · ←→ 切换问题 · ↑↓ 移动 · Esc 取消", style="dim")

        border_style = "#f59e0b" if self.has_focus else "cyan"
        return Panel(text, title="请回答以下问题", border_style=border_style, padding=(0, 1))

    def on_focus(self) -> None:
        self.refresh()

    def on_blur(self) -> None:
        self.refresh()

    def _is_answered(self, q_idx: int) -> bool:
        ans = self.answers.get(q_idx)
        if isinstance(ans, set):
            return len(ans) > 0
        if isinstance(ans, int):
            return ans >= 0
        return False

    def _is_option_checked(self, q_idx: int, opt_idx: int) -> bool:
        ans = self.answers.get(q_idx)
        if isinstance(ans, set):
            return opt_idx in ans
        if isinstance(ans, int):
            return ans == opt_idx
        return False

    def _is_custom_selected(self, q_idx: int) -> bool:
        ans = self.answers.get(q_idx)
        custom_idx = len(self.questions[q_idx].options)
        if isinstance(ans, set):
            return custom_idx in ans
        if isinstance(ans, int):
            return ans == custom_idx
        return False

    def action_prev_question(self) -> None:
        if self.typing_mode:
            return
        if self.current_question > 0:
            self.current_question -= 1
            self.current_option = 0
            self.refresh()

    def action_next_question(self) -> None:
        if self.typing_mode:
            return
        if self.current_question < len(self.questions) - 1:
            self.current_question += 1
            self.current_option = 0
            self.refresh()

    def action_prev_option(self) -> None:
        if self.typing_mode:
            return
        if self.current_option > 0:
            self.current_option -= 1
            self.refresh()

    def action_next_option(self) -> None:
        if self.typing_mode:
            return
        q = self.questions[self.current_question]
        max_option = len(q.options)
        if self.current_option < max_option:
            self.current_option += 1
            self.refresh()

    def action_select_option(self) -> None:
        if self.typing_mode:
            self._confirm_typing()
            return

        q = self.questions[self.current_question]
        custom_idx = len(q.options)

        if self.current_option == custom_idx:
            self.typing_mode = True
            self.typing_buffer = self.custom_inputs.get(self.current_question, "")
            self.refresh()
            return

        if q.multi_select:
            ans = self.answers[self.current_question]
            if isinstance(ans, set):
                if self.current_option in ans:
                    ans.remove(self.current_option)
                else:
                    ans.add(self.current_option)
        else:
            self.answers[self.current_question] = self.current_option
        self.refresh()

    def _confirm_typing(self) -> None:
        q = self.questions[self.current_question]
        custom_idx = len(q.options)
        value = self.typing_buffer.strip()

        self.custom_inputs[self.current_question] = value
        self.typing_mode = False

        if q.multi_select:
            ans = self.answers[self.current_question]
            if isinstance(ans, set):
                if value:
                    ans.add(custom_idx)
                else:
                    ans.discard(custom_idx)
        else:
            self.answers[self.current_question] = custom_idx if value else -1
        self.refresh()

    def on_key(self, event) -> None:
        if not self.typing_mode:
            if event.key == "s" and all(self._is_answered(i) for i in range(len(self.questions))):
                self._submit()
            return

        if event.key == "escape":
            self.typing_mode = False
            self.refresh()
            return

        if event.key == "backspace":
            self.typing_buffer = self.typing_buffer[:-1]
            self.refresh()
            return

        if event.character and len(event.character) == 1:
            self.typing_buffer += event.character
            self.refresh()

    def action_cancel(self) -> None:
        self.post_message(self.Cancelled(self.request_id))

    def _submit(self) -> None:
        answers_out: dict[str, str | list[str]] = {}
        for i, q in enumerate(self.questions):
            ans = self.answers.get(i)
            if isinstance(ans, set):
                selected: list[str] = []
                for idx in sorted(ans):
                    if idx < len(q.options):
                        selected.append(q.options[idx].label)
                    else:
                        selected.append(self.custom_inputs.get(i, ""))
                answers_out[q.header] = [v for v in selected if v]
            elif isinstance(ans, int):
                if ans < 0:
                    continue
                if ans < len(q.options):
                    answers_out[q.header] = q.options[ans].label
                else:
                    answers_out[q.header] = self.custom_inputs.get(i, "")
        self.post_message(self.Submitted(self.request_id, answers_out))



from __future__ import annotations

import asyncio
from dataclasses import dataclass
from uuid import uuid4

from minicc.core.events import AskUserRequested
from minicc.core.models import AskUserResponse, UserCancelledError


@dataclass
class AskUserResult:
    submitted: bool
    answers: dict[str, str | list[str]]


class AskUserService:
    def __init__(self, event_bus) -> None:
        self._bus = event_bus
        self._pending: dict[str, asyncio.Future[AskUserResult]] = {}

    async def ask(self, questions) -> AskUserResult:
        request_id = uuid4().hex[:8]
        fut: asyncio.Future[AskUserResult] = asyncio.get_running_loop().create_future()
        self._pending[request_id] = fut
        self._bus.emit(AskUserRequested(request_id=request_id, questions=questions))
        result = await fut
        if not result.submitted:
            raise UserCancelledError("用户取消了操作")
        return result

    def resolve(self, request_id: str, submitted: bool, answers: dict[str, str | list[str]]) -> None:
        fut = self._pending.pop(request_id, None)
        if fut is None or fut.done():
            return
        fut.set_result(AskUserResult(submitted=submitted, answers=answers))




from __future__ import annotations

import asyncio

from pydantic_ai import RunContext

from minicc.core.events import TodoUpdated
from minicc.core.models import MiniCCDeps, TodoItem, ToolResult


async def task(
    ctx: RunContext[MiniCCDeps],
    prompt: str,
    description: str,
    subagent_type: str = "general-purpose",
    wait: bool = True,
) -> ToolResult:
    """
    运行子代理任务。

    - 默认 wait=True：等待子代理完成后返回结果文本（便于主 Agent 整合后继续推理）。
    - wait=False：后台启动并立即返回 task_id，可用 wait_subagents 再统一等待与汇总。
    """
    service = ctx.deps.subagent_service
    if service is None:
        return ToolResult(success=False, output="", error="subagent_service 未初始化")

    task_id, result = await service.run(
        prompt=prompt,
        description=description,
        subagent_type=subagent_type,
        background=not wait,
    )

    if not wait:
        return ToolResult(success=True, output=f"已创建子任务 [{task_id}]: {description}")

    if result is None:
        return ToolResult(success=True, output=f"子任务 [{task_id}] 已完成（无输出）")

    return ToolResult(success=True, output=f"子任务 [{task_id}] 结果：\n{result}")


async def wait_subagents(ctx: RunContext[MiniCCDeps]) -> ToolResult:
    """
    等待当前所有后台子任务执行完毕，并返回汇总结果。

    适用于曾用 task(wait=False) 启动了多个子任务的场景。
    """
    tasks = list(ctx.deps.sub_agent_tasks.values())
    if not tasks:
        return ToolResult(success=True, output="当前没有运行中的子任务")

    await asyncio.gather(*tasks, return_exceptions=True)

    lines: list[str] = []
    for task_id, t in ctx.deps.sub_agents.items():
        if t.status in ("completed", "failed"):
            head = f"[{task_id}] {t.description or ''}".strip()
            lines.append(f"{head} ({t.status})")
            if t.result:
                lines.append(t.result)
                lines.append("")

    output = "\n".join(lines).rstrip() or "子任务已结束（无可用结果）"
    return ToolResult(success=True, output=output)


async def todo_write(ctx: RunContext[MiniCCDeps], todos: list[dict[str, str]]) -> ToolResult:
    try:
        new_todos: list[TodoItem] = []
        for item in todos:
            new_todos.append(
                TodoItem(
                    content=item.get("content", ""),
                    status=item.get("status", "pending"),
                    active_form=item.get("activeForm", item.get("active_form", "")),
                )
            )

        ctx.deps.todos = new_todos
        if ctx.deps.event_bus is not None:
            ctx.deps.event_bus.emit(TodoUpdated(todos=new_todos))

        summary_lines = []
        for todo in new_todos:
            status_icon = {"pending": "⏳", "in_progress": "🔄", "completed": "✅"}.get(todo.status, "?")
            summary_lines.append(f"{status_icon} {todo.content}")

        return ToolResult(success=True, output=f"已更新 {len(new_todos)} 个任务\n" + "\n".join(summary_lines))
    except Exception as e:
        return ToolResult(success=False, output="", error=str(e))



"""
文件引用面板（@ 引用文件）

用于在输入框中输入 `@` 时展示文件候选列表，并允许键盘选择插入路径。
"""

from __future__ import annotations

from rich.panel import Panel
from rich.text import Text
from textual.widgets import Static


class FileMentionPanel(Static):
    def __init__(self, query: str, items: list[str], selected: int = 0, **kwargs):
        self.query = query
        self.items = items
        self.selected = selected
        super().__init__(**kwargs)

    def update_items(self, query: str, items: list[str], selected: int = 0) -> None:
        self.query = query
        self.items = items
        self.selected = selected
        self.refresh()

    def render(self) -> Panel:
        text = Text()
        text.append("@", style="bold cyan")
        text.append(self.query or "", style="cyan")
        text.append("  选择文件（↑↓ 选择，Enter/Tab 插入，Esc 关闭）\n", style="dim")

        if not self.items:
            text.append("（无匹配文件）", style="dim")
            return Panel(text, title="文件引用", border_style="cyan", padding=(0, 1))

        for i, p in enumerate(self.items[:20]):
            is_sel = i == self.selected
            text.append("❯ " if is_sel else "  ", style="bold yellow" if is_sel else "dim")
            text.append(p, style="bold yellow" if is_sel else "")
            text.append("\n")

        if len(self.items) > 20:
            text.append(f"... 还有 {len(self.items) - 20} 个结果", style="dim")

        return Panel(text, title="文件引用", border_style="cyan", padding=(0, 1))




from __future__ import annotations

import pytest
from textual.app import App, ComposeResult

from minicc.tui.chat_input import ChatInput


class _ChatInputTestApp(App):
    def __init__(self):
        super().__init__()
        self.submitted: str | None = None

    def compose(self) -> ComposeResult:
        yield ChatInput(id="input", soft_wrap=True, show_line_numbers=False)

    def on_mount(self) -> None:
        self.query_one(ChatInput).focus()

    def on_chat_input_submitted(self, event: ChatInput.Submitted) -> None:
        self.submitted = event.value


@pytest.mark.asyncio
async def test_ctrl_j_inserts_newline_and_moves_cursor():
    app = _ChatInputTestApp()
    async with app.run_test() as pilot:
        await pilot.press("h", "i", "ctrl+j", "t", "h", "e", "r", "e")
        widget = app.query_one(ChatInput)
        assert widget.text == "hi\nthere"
        assert widget.cursor_location == (1, 5)


@pytest.mark.asyncio
async def test_enter_submits_without_inserting_newline():
    app = _ChatInputTestApp()
    async with app.run_test() as pilot:
        await pilot.press("h", "i", "ctrl+j", "t", "h", "e", "r", "e")
        await pilot.press("enter")
        await pilot.pause()

        widget = app.query_one(ChatInput)
        assert app.submitted == "hi\nthere"
        assert widget.text == "hi\nthere"


@pytest.mark.asyncio
async def test_mention_handler_can_intercept_enter():
    app = _ChatInputTestApp()
    async with app.run_test() as pilot:
        widget = app.query_one(ChatInput)
        widget.set_mention_key_handler(lambda key: key == "enter")

        await pilot.press("h", "i")
        await pilot.press("enter")
        await pilot.pause()

        assert app.submitted is None
        assert widget.text == "hi"




from __future__ import annotations

from pydantic_ai import Agent

from minicc.core.models import MiniCCDeps
from minicc.tools.file import edit_file, read_file, write_file
from minicc.tools.interact import ask_user
from minicc.tools.search import glob_files, grep_search
from minicc.tools.shell import bash, bash_output, kill_shell
from minicc.tools.task import task, todo_write, wait_subagents


def register_tools(agent: Agent[MiniCCDeps, str]) -> None:
    # 文件操作
    agent.tool(read_file)
    agent.tool(write_file)
    agent.tool(edit_file)

    # 搜索
    agent.tool(glob_files)
    agent.tool(grep_search)

    # 命令行
    agent.tool(bash)
    agent.tool(bash_output)
    agent.tool(kill_shell)

    # 任务与交互
    agent.tool(task)
    agent.tool(todo_write)
    agent.tool(wait_subagents)
    agent.tool(ask_user)



from __future__ import annotations

import asyncio
from dataclasses import dataclass

import pytest

from minicc.core.events import EventBus
from minicc.core.models import Config, MiniCCDeps, Provider
from minicc.core.services.subagents import SubAgentService
from minicc.tools.task import task as task_tool
from minicc.tools.task import wait_subagents


@dataclass
class _DummyResult:
    output: str


class _DummyAgent:
    async def run(self, prompt: str, deps=None):
        await asyncio.sleep(0.01)
        return _DummyResult(output=f"done:{prompt}")


@dataclass
class _Ctx:
    deps: MiniCCDeps


def _deps() -> MiniCCDeps:
    cfg = Config(provider=Provider.ANTHROPIC, model="test-model", api_key="test-key")
    return MiniCCDeps(config=cfg, cwd="/tmp/minicc-test", fs=None)


@pytest.mark.asyncio
async def test_wait_subagents_no_tasks_returns_message():
    deps = _deps()
    ctx = _Ctx(deps=deps)
    result = await wait_subagents(ctx)
    assert result.success is True
    assert "没有" in result.output


@pytest.mark.asyncio
async def test_task_wait_false_then_wait_subagents_returns_summary():
    deps = _deps()
    bus = EventBus()

    service = SubAgentService(
        deps=deps,
        event_bus=bus,
        agent_factory=lambda: _DummyAgent(),
    )
    deps.subagent_service = service

    ctx = _Ctx(deps=deps)

    r1 = await task_tool(ctx, prompt="P1", description="D1", wait=False)
    r2 = await task_tool(ctx, prompt="P2", description="D2", wait=False)
    assert r1.success and r2.success
    assert "已创建子任务" in r1.output

    summary = await wait_subagents(ctx)
    assert summary.success is True
    assert "D1" in summary.output
    assert "D2" in summary.output
    assert "(completed)" in summary.output
    assert "done:P1" in summary.output
    assert "done:P2" in summary.output




from __future__ import annotations

import difflib
from pathlib import Path

from minicc.core.models import DiffLine

DEFAULT_READ_LIMIT = 2000
MAX_OUTPUT_CHARS = 30000
DEFAULT_BASH_TIMEOUT_MS = 120000


def resolve_path(cwd: str, path: str) -> Path:
    p = Path(path)
    if p.is_absolute():
        return p
    return Path(cwd) / p


def normalize_whitespace(text: str) -> str:
    text = text.replace("\t", "    ")
    lines = [line.rstrip() for line in text.split("\n")]
    return "\n".join(lines)


def find_whitespace_tolerant(content: str, normalized_pattern: str) -> str | None:
    content_lines = content.split("\n")
    pattern_lines = normalized_pattern.split("\n")
    pattern_len = len(pattern_lines)

    for i in range(len(content_lines) - pattern_len + 1):
        window = content_lines[i : i + pattern_len]
        normalized_window = [normalize_whitespace(line) for line in window]
        if "\n".join(normalized_window) == normalized_pattern:
            return "\n".join(window)
    return None


def generate_unified_diff(old: str, new: str, filename: str = "") -> str:
    old_lines = old.splitlines(keepends=True)
    new_lines = new.splitlines(keepends=True)
    diff = difflib.unified_diff(
        old_lines,
        new_lines,
        fromfile=f"a/{filename}" if filename else "a",
        tofile=f"b/{filename}" if filename else "b",
    )
    return "".join(diff)


def generate_diff_lines(old: str, new: str) -> list[DiffLine]:
    old_lines = old.splitlines(keepends=True)
    new_lines = new.splitlines(keepends=True)
    diff = difflib.unified_diff(old_lines, new_lines, lineterm="")
    result: list[DiffLine] = []

    for line in diff:
        if line.startswith(("+++", "---", "@@")):
            continue
        if line.startswith("+"):
            result.append(DiffLine(type="add", content=line[1:].rstrip("\n")))
        elif line.startswith("-"):
            result.append(DiffLine(type="remove", content=line[1:].rstrip("\n")))
        else:
            result.append(DiffLine(type="context", content=line.rstrip("\n")))
    return result


def format_diff_lines(diff_lines: list[DiffLine]) -> str:
    lines: list[str] = []
    for line in diff_lines:
        prefix = {"add": "+", "remove": "-", "context": " "}.get(line.type, " ")
        lines.append(f"{prefix}{line.content}")
    return "\n".join(lines)




from __future__ import annotations

import os
from dataclasses import dataclass

from agent_gear import FileSystem

from minicc.core.agent import create_agent
from minicc.core.events import EventBus
from minicc.core.mcp import load_mcp_toolsets
from minicc.core.models import Config, MiniCCDeps
from minicc.core.services.ask_user import AskUserService
from minicc.core.services.subagents import SubAgentService
from minicc.tools import register_tools


@dataclass
class MiniCCRuntime:
    config: Config
    cwd: str
    deps: MiniCCDeps
    agent: object
    event_bus: EventBus
    fs: FileSystem
    toolsets: list

    def close(self) -> None:
        try:
            self.fs.close()
        except Exception:
            pass


def build_runtime(config: Config | None = None, cwd: str | None = None) -> MiniCCRuntime:
    cwd = cwd or os.getcwd()
    cfg = config or Config()

    event_bus: EventBus = EventBus()
    toolsets = load_mcp_toolsets(cwd)

    fs = FileSystem(cwd, auto_watch=True)

    deps = MiniCCDeps(config=cfg, cwd=cwd, fs=fs)
    deps.event_bus = event_bus
    deps.ask_user_service = AskUserService(event_bus)

    def _subagent_factory():
        return create_agent(cfg, cwd=cwd, toolsets=toolsets, register_tools=register_tools)

    deps.subagent_service = SubAgentService(deps=deps, event_bus=event_bus, agent_factory=_subagent_factory)

    agent = create_agent(cfg, cwd=cwd, toolsets=toolsets, register_tools=register_tools)
    return MiniCCRuntime(config=cfg, cwd=cwd, deps=deps, agent=agent, event_bus=event_bus, fs=fs, toolsets=toolsets)




from __future__ import annotations

from dataclasses import dataclass

import pytest

import minicc.core.runtime as runtime
from minicc.core.models import Config, Provider


@dataclass
class _DummyFS:
    cwd: str
    auto_watch: bool = True

    def close(self) -> None:
        return None


def test_build_runtime_preloads_mcp_and_passes_toolsets(monkeypatch):
    calls: list[tuple[str, object]] = []
    toolsets = ["ts-1"]

    monkeypatch.setattr(runtime, "load_mcp_toolsets", lambda cwd: toolsets)
    monkeypatch.setattr(runtime, "FileSystem", lambda cwd, auto_watch=True: _DummyFS(cwd=cwd, auto_watch=auto_watch))

    def fake_create_agent(cfg, cwd, toolsets, register_tools):
        calls.append((cwd, toolsets))
        return object()

    monkeypatch.setattr(runtime, "create_agent", fake_create_agent)

    cfg = Config(provider=Provider.ANTHROPIC, model="test-model", api_key="test-key")
    rt = runtime.build_runtime(config=cfg, cwd="/tmp/minicc-test")
    assert rt.toolsets == toolsets
    assert calls == [("/tmp/minicc-test", toolsets)]

    # 子代理 factory 也必须复用启动时预加载的 toolsets（不再懒加载）
    assert rt.deps.subagent_service is not None
    rt.deps.subagent_service.agent_factory()
    assert calls == [("/tmp/minicc-test", toolsets), ("/tmp/minicc-test", toolsets)]



from __future__ import annotations

import pytest

from minicc.tools.interact import _validate_and_normalize_ask_user_questions


def test_validate_rejects_list_of_strings():
    with pytest.raises(ValueError, match="ask_user 参数错误"):
        _validate_and_normalize_ask_user_questions(["Q1", "Q2"])  # type: ignore[arg-type]


def test_validate_requires_header_and_options():
    with pytest.raises(ValueError, match="header"):
        _validate_and_normalize_ask_user_questions(
            [{"question": "Q1", "header": "  ", "options": [{"label": "A"}]}]
        )

    with pytest.raises(ValueError, match="options"):
        _validate_and_normalize_ask_user_questions(
            [{"question": "Q1", "header": "H1", "options": []}]
        )


def test_normalize_makes_duplicate_headers_unique():
    questions = [
        {"question": "Q1", "header": "H", "options": [{"label": "A"}]},
        {"question": "Q2", "header": "H", "options": [{"label": "B"}]},
    ]

    normalized = _validate_and_normalize_ask_user_questions(questions)
    assert [q.header for q in normalized] == ["H#1", "H#2"]


def test_normalize_collapses_header_whitespace():
    questions = [{"question": "Q1", "header": "  A   B  ", "options": [{"label": "A"}]}]
    normalized = _validate_and_normalize_ask_user_questions(questions)
    assert normalized[0].header == "A B"


def test_normalize_falls_back_question_to_header():
    questions = [{"question": " ", "header": "H1", "options": [{"label": "A"}]}]
    normalized = _validate_and_normalize_ask_user_questions(questions)
    assert normalized[0].question == "H1"



"""
MiniCC - 极简教学版 AI 编程助手

基于 pydantic-ai 和 textual 实现的轻量级 Claude Code 替代品，
用于学习 AI Agent 的核心实现原理。

基本用法:
    $ minicc              # 启动 TUI 界面
    $ python -m minicc    # 等效启动方式

编程接口:
    from minicc import MiniCCApp, create_agent

    # 使用应用
    app = MiniCCApp()
    app.run()

    # 直接使用 Agent
    agent = create_agent(config)
    result = await agent.run("你的问题")

配置:
    配置文件位于 ~/.minicc/config.json
    系统提示词位于 ~/.minicc/AGENTS.md
"""

__version__ = "0.3.2"
__author__ = "MiniCC Contributors"

# 仅对外暴露 TUI 入口
from minicc.cli import main
from minicc.tui.app import MiniCCApp

__all__ = ["__version__", "__author__", "MiniCCApp", "main"]



from __future__ import annotations

import asyncio
from dataclasses import dataclass
from typing import Any, AsyncIterator, Generic, TypeVar

T = TypeVar("T")


class EventBus(Generic[T]):
    """简单的单消费者事件总线（asyncio.Queue）。"""

    def __init__(self) -> None:
        self._queue: asyncio.Queue[T] = asyncio.Queue()

    def emit(self, event: T) -> None:
        self._queue.put_nowait(event)

    async def next(self) -> T:
        return await self._queue.get()

    async def iter(self) -> AsyncIterator[T]:
        while True:
            yield await self.next()


@dataclass(frozen=True)
class ToolCallStarted:
    tool_call_id: str
    tool_name: str
    args: dict[str, Any] | None


@dataclass(frozen=True)
class ToolCallFinished:
    tool_call_id: str
    tool_name: str
    ok: bool
    content: Any
    error: str | None = None


@dataclass(frozen=True)
class TodoUpdated:
    todos: list[Any]


@dataclass(frozen=True)
class AskUserRequested:
    request_id: str
    questions: list[Any]


@dataclass(frozen=True)
class AskUserResolved:
    request_id: str
    submitted: bool
    answers: dict[str, str | list[str]]


@dataclass(frozen=True)
class SubAgentCreated:
    task_id: str
    description: str
    prompt: str


@dataclass(frozen=True)
class SubAgentUpdated:
    task_id: str
    status: str
    result: str | None = None



from __future__ import annotations

import asyncio

import pytest

from minicc.core.events import AskUserRequested, EventBus
from minicc.core.models import Question, QuestionOption, UserCancelledError
from minicc.core.services.ask_user import AskUserService


@pytest.mark.asyncio
async def test_ask_user_emits_event_and_resolves():
    bus: EventBus = EventBus()
    service = AskUserService(bus)

    questions = [
        Question(
            question="Q1",
            header="H1",
            options=[QuestionOption(label="A")],
            multi_select=False,
        )
    ]

    async def runner():
        return await service.ask(questions)

    t = asyncio.create_task(runner())
    ev = await asyncio.wait_for(bus.next(), timeout=1)
    assert isinstance(ev, AskUserRequested)
    assert ev.questions[0].header == "H1"

    service.resolve(ev.request_id, submitted=True, answers={"H1": "A"})
    result = await asyncio.wait_for(t, timeout=1)
    assert result.answers == {"H1": "A"}


@pytest.mark.asyncio
async def test_ask_user_cancel_raises():
    bus: EventBus = EventBus()
    service = AskUserService(bus)

    questions = [
        Question(
            question="Q1",
            header="H1",
            options=[QuestionOption(label="A")],
            multi_select=False,
        )
    ]

    async def runner():
        await service.ask(questions)

    t = asyncio.create_task(runner())
    ev = await asyncio.wait_for(bus.next(), timeout=1)
    assert isinstance(ev, AskUserRequested)

    service.resolve(ev.request_id, submitted=False, answers={})
    with pytest.raises(UserCancelledError):
        await asyncio.wait_for(t, timeout=1)




from __future__ import annotations

import os
import re
from pathlib import Path
from typing import Literal

from pydantic_ai import RunContext

from minicc.core.models import MiniCCDeps, ToolResult
from minicc.tools.common import MAX_OUTPUT_CHARS, resolve_path


async def glob_files(
    ctx: RunContext[MiniCCDeps],
    pattern: str,
    path: str | None = None,
) -> ToolResult:
    base = resolve_path(ctx.deps.cwd, path or ".")
    if not base.exists():
        return ToolResult(success=False, output="", error=f"路径不存在: {path or '.'}")

    fs = ctx.deps.fs
    try:
        if fs is not None:
            full_pattern = f"{path}/{pattern}" if path else pattern
            matches = fs.glob(full_pattern)
            if not matches:
                return ToolResult(success=True, output=f"未找到匹配 '{pattern}' 的文件")
            return ToolResult(success=True, output="\n".join(matches))

        from wcmatch import glob as wcglob

        flags = wcglob.GLOBSTAR | wcglob.BRACE | wcglob.EXTGLOB
        matches = wcglob.glob(pattern, root_dir=str(base), flags=flags)
        if not matches:
            return ToolResult(success=True, output=f"未找到匹配 '{pattern}' 的文件")
        rels = [os.path.relpath(m, start=str(base)) for m in matches]
        return ToolResult(success=True, output="\n".join(rels))
    except Exception as e:
        return ToolResult(success=False, output="", error=str(e))


async def grep_search(
    ctx: RunContext[MiniCCDeps],
    pattern: str,
    path: str | None = None,
    glob: str | None = None,
    output_mode: Literal["content", "files_with_matches", "count"] = "files_with_matches",
    context_before: int | None = None,
    context_after: int | None = None,
    context: int | None = None,
    case_insensitive: bool = False,
    head_limit: int | None = None,
    file_type: str | None = None,
) -> ToolResult:
    search_path = resolve_path(ctx.deps.cwd, path or ".")
    if not search_path.exists():
        return ToolResult(success=False, output="", error=f"路径不存在: {path or '.'}")

    try:
        from ripgrepy import Ripgrepy

        rg = Ripgrepy(pattern, str(search_path))
        if case_insensitive:
            rg = rg.i()
        if glob:
            rg = rg.glob(glob)
        if file_type:
            rg = rg.type(file_type)
        if context:
            rg = rg.context(context)
        else:
            if context_before:
                rg = rg.before_context(context_before)
            if context_after:
                rg = rg.after_context(context_after)

        if output_mode == "files_with_matches":
            rg = rg.files_with_matches()
        elif output_mode == "count":
            rg = rg.count()
        else:
            rg = rg.with_filename().line_number()

        result = rg.run()
        output = result.as_string if hasattr(result, "as_string") else str(result)
        if not output.strip():
            return ToolResult(success=True, output=f"未找到匹配 '{pattern}' 的内容")

        lines = output.strip().split("\n")
        if head_limit and len(lines) > head_limit:
            lines = lines[:head_limit]
            output = "\n".join(lines) + "\n... 还有更多结果"
        else:
            output = "\n".join(lines)

        if len(output) > MAX_OUTPUT_CHARS:
            output = output[:MAX_OUTPUT_CHARS] + "\n... 输出已截断"
        return ToolResult(success=True, output=output)

    except ImportError:
        return await _grep_fallback(
            ctx,
            pattern=pattern,
            search_path=search_path,
            glob=glob,
            output_mode=output_mode,
            case_insensitive=case_insensitive,
            head_limit=head_limit,
        )
    except Exception as e:
        return ToolResult(success=False, output="", error=str(e))


async def _grep_fallback(
    ctx: RunContext[MiniCCDeps],
    *,
    pattern: str,
    search_path: Path,
    glob: str | None,
    output_mode: str,
    case_insensitive: bool,
    head_limit: int | None,
) -> ToolResult:
    flags = re.IGNORECASE if case_insensitive else 0
    try:
        regex = re.compile(pattern, flags=flags)
    except re.error as e:
        return ToolResult(success=False, output="", error=f"无效正则: {e}")

    matched_files: list[str] = []
    matched_lines: list[str] = []
    file_counts: dict[str, int] = {}

    for p in search_path.rglob("*"):
        if not p.is_file():
            continue
        if glob and not p.match(glob):
            continue
        try:
            text = p.read_text(encoding="utf-8")
        except Exception:
            continue
        hits = 0
        for i, line in enumerate(text.splitlines(), start=1):
            if regex.search(line):
                hits += 1
                if output_mode == "content":
                    matched_lines.append(f"{p}:{i}:{line}")
        if hits:
            matched_files.append(str(p))
            file_counts[str(p)] = hits

    if output_mode == "files_with_matches":
        output = "\n".join(matched_files)
    elif output_mode == "count":
        output = "\n".join(f"{f}:{c}" for f, c in file_counts.items())
    else:
        output = "\n".join(matched_lines)

    if not output.strip():
        return ToolResult(success=True, output=f"未找到匹配 '{pattern}' 的内容")

    if head_limit:
        lines = output.split("\n")
        if len(lines) > head_limit:
            output = "\n".join(lines[:head_limit]) + "\n... 还有更多结果"

    if len(output) > MAX_OUTPUT_CHARS:
        output = output[:MAX_OUTPUT_CHARS] + "\n... 输出已截断"

    return ToolResult(success=True, output=output)




from __future__ import annotations

import asyncio
from dataclasses import dataclass
from typing import Any, Callable
from uuid import uuid4

from pydantic_ai import Agent

from minicc.core.events import SubAgentCreated, SubAgentUpdated
from minicc.core.models import AgentTask, MiniCCDeps


@dataclass
class SubAgentService:
    deps: MiniCCDeps
    event_bus: Any
    agent_factory: Callable[[], Agent[MiniCCDeps, str]]

    async def run(
        self,
        *,
        prompt: str,
        description: str,
        subagent_type: str = "general-purpose",
        background: bool = False,
    ) -> tuple[str, str | None]:
        """
        运行子代理任务。

        - background=True：后台启动，立即返回 task_id。
        - background=False：等待执行完成后返回 (task_id, result)。
        """
        task_id = uuid4().hex[:8]
        task_obj = AgentTask(
            task_id=task_id,
            description=description,
            prompt=prompt,
            subagent_type=subagent_type,
            status="pending",
        )

        self.deps.sub_agents[task_id] = task_obj
        self.event_bus.emit(SubAgentCreated(task_id=task_id, description=description, prompt=prompt))
        self.event_bus.emit(SubAgentUpdated(task_id=task_id, status="pending"))

        if background:
            handle = asyncio.create_task(self._run(task_obj))
            self.deps.sub_agent_tasks[task_id] = handle
            return task_id, None

        await self._run(task_obj)
        return task_id, task_obj.result

    async def _run(self, task_obj: AgentTask) -> None:
        task_obj.status = "running"
        self.event_bus.emit(SubAgentUpdated(task_id=task_obj.task_id, status="running"))

        try:
            agent = self.agent_factory()
            result = await agent.run(task_obj.prompt, deps=self.deps)
            task_obj.status = "completed"
            task_obj.result = getattr(result, "output", str(result))
            self.event_bus.emit(
                SubAgentUpdated(task_id=task_obj.task_id, status="completed", result=task_obj.result)
            )
        except Exception as e:
            task_obj.status = "failed"
            task_obj.result = str(e)
            self.event_bus.emit(SubAgentUpdated(task_id=task_obj.task_id, status="failed", result=task_obj.result))
        finally:
            self.deps.sub_agent_tasks.pop(task_obj.task_id, None)



from __future__ import annotations

from pathlib import Path
from typing import Any

from pydantic_ai import RunContext

from minicc.core.models import MiniCCDeps, ToolResult
from minicc.tools.common import (
    DEFAULT_READ_LIMIT,
    find_whitespace_tolerant,
    generate_unified_diff,
    normalize_whitespace,
    resolve_path,
)


async def read_file(
    ctx: RunContext[MiniCCDeps],
    file_path: str,
    offset: int | None = None,
    limit: int | None = None,
) -> ToolResult:
    fs = ctx.deps.fs
    resolved = resolve_path(ctx.deps.cwd, file_path)

    try:
        if not resolved.exists():
            return ToolResult(success=False, output="", error=f"文件不存在: {file_path}")
        if resolved.is_dir():
            return ToolResult(success=False, output="", error=f"不是文件: {file_path}")

        start_line = (offset or 1) - 1
        count = limit or DEFAULT_READ_LIMIT

        if fs is not None:
            rel_path = (
                str(resolved.relative_to(ctx.deps.cwd))
                if resolved.is_absolute()
                else file_path
            )
            lines = fs.read_lines(rel_path, start_line=start_line, count=count)
        else:
            text = resolved.read_text(encoding="utf-8")
            all_lines = text.splitlines()
            lines = all_lines[start_line : start_line + count]

        if not lines:
            return ToolResult(success=True, output="（文件为空或偏移超出范围）")

        formatted: list[str] = []
        for i, line in enumerate(lines, start=start_line + 1):
            if len(line) > 2000:
                line = line[:2000] + "..."
            formatted.append(f"{i:6}\t{line}")

        output = "\n".join(formatted)
        if len(lines) >= count:
            output += "\n\n... 可能还有更多行未显示"
        return ToolResult(success=True, output=output)

    except UnicodeDecodeError:
        return ToolResult(success=False, output="", error="无法读取文件：可能是二进制文件")
    except Exception as e:
        return ToolResult(success=False, output="", error=str(e))


async def write_file(
    ctx: RunContext[MiniCCDeps],
    file_path: str,
    content: str,
) -> ToolResult:
    fs = ctx.deps.fs
    resolved = resolve_path(ctx.deps.cwd, file_path)

    try:
        resolved.parent.mkdir(parents=True, exist_ok=True)

        if fs is not None:
            rel_path = (
                str(resolved.relative_to(ctx.deps.cwd))
                if resolved.is_absolute()
                else file_path
            )
            ok = fs.write_file(rel_path, content)
            if not ok:
                return ToolResult(success=False, output="", error="写入失败")
        else:
            resolved.write_text(content, encoding="utf-8")

        return ToolResult(success=True, output=f"已写入文件: {file_path} ({len(content)} 字符)")
    except Exception as e:
        return ToolResult(success=False, output="", error=str(e))


async def edit_file(
    ctx: RunContext[MiniCCDeps],
    file_path: str,
    old_string: str,
    new_string: str,
    replace_all: bool = False,
) -> ToolResult:
    fs = ctx.deps.fs
    resolved = resolve_path(ctx.deps.cwd, file_path)

    try:
        if not resolved.exists():
            return ToolResult(success=False, output="", error=f"文件不存在: {file_path}")
        if resolved.is_dir():
            return ToolResult(success=False, output="", error=f"不是文件: {file_path}")
        if old_string == new_string:
            return ToolResult(success=False, output="", error="new_string 必须与 old_string 不同")

        if fs is not None:
            rel_path = (
                str(resolved.relative_to(ctx.deps.cwd))
                if resolved.is_absolute()
                else file_path
            )
            current = fs.read_file(rel_path)
        else:
            current = resolved.read_text(encoding="utf-8")

        exact_count = current.count(old_string)
        actual_old = old_string

        if exact_count == 0:
            normalized_old = normalize_whitespace(old_string)
            match = find_whitespace_tolerant(current, normalized_old)
            if match is None:
                return ToolResult(
                    success=False,
                    output="",
                    error="未找到要替换的内容，请确保 old_string 精确匹配文件内容",
                )
            actual_old = match
            exact_count = 1

        if exact_count > 1 and not replace_all:
            return ToolResult(
                success=False,
                output="",
                error=f"old_string 在文件中出现了 {exact_count} 次，请设置 replace_all=True 或提供更精确的 old_string",
            )

        if replace_all:
            updated = current.replace(actual_old, new_string)
        else:
            updated = current.replace(actual_old, new_string, 1)

        if fs is not None:
            rel_path = (
                str(resolved.relative_to(ctx.deps.cwd))
                if resolved.is_absolute()
                else file_path
            )
            ok = fs.write_file(rel_path, updated)
            if not ok:
                return ToolResult(success=False, output="", error="写入失败")
        else:
            resolved.write_text(updated, encoding="utf-8")

        diff = generate_unified_diff(current, updated, filename=str(resolved))
        return ToolResult(success=True, output=diff or "OK")

    except Exception as e:
        return ToolResult(success=False, output="", error=str(e))




"""
MiniCC 数据模型定义

注意：该项目允许 breaking change，模型仅服务于内部（TUI）与工具实现。
"""

from __future__ import annotations

from dataclasses import dataclass, field
from enum import Enum
from typing import Any, Literal, Optional

from pydantic import BaseModel, Field


class Provider(str, Enum):
    """LLM 提供商枚举"""

    ANTHROPIC = "anthropic"
    OPENAI = "openai"


class PromptCache(BaseModel):
    """
    Anthropic Prompt Cache 配置

    每个字段支持 bool（True=5m TTL）或 '5m'/'1h'。
    """

    instructions: bool | Literal["5m", "1h"] = False
    messages: bool | Literal["5m", "1h"] = False
    tool_definitions: bool | Literal["5m", "1h"] = False


class Config(BaseModel):
    """应用配置结构（~/.minicc/config.json）。"""

    provider: Provider = Provider.ANTHROPIC
    model: str = "claude-sonnet-4-20250514"
    api_key: Optional[str] = None
    base_url: Optional[str] = None
    prompt_cache: PromptCache = Field(default_factory=PromptCache)


class ToolResult(BaseModel):
    """工具执行结果（供模型与 UI 渲染）。"""

    success: bool
    output: str
    error: Optional[str] = None


class DiffLine(BaseModel):
    type: str  # "add" | "remove" | "context"
    content: str
    line_no: Optional[int] = None


class AgentTask(BaseModel):
    task_id: str
    description: str = ""
    prompt: str
    subagent_type: str = "general-purpose"
    status: str = "pending"  # pending, running, completed, failed
    result: Optional[str] = None


class TodoItem(BaseModel):
    content: str
    status: Literal["pending", "in_progress", "completed"]
    active_form: str


class BackgroundShell(BaseModel):
    shell_id: str
    command: str
    description: str = ""
    output_buffer: str = ""
    is_running: bool = True


class QuestionOption(BaseModel):
    label: str
    description: str = ""


class Question(BaseModel):
    question: str
    header: str
    options: list[QuestionOption]
    multi_select: bool = False


class AskUserRequest(BaseModel):
    questions: list[Question]


class AskUserResponse(BaseModel):
    submitted: bool
    answers: dict[str, str | list[str]]


class UserCancelledError(Exception):
    pass


@dataclass
class MiniCCDeps:
    """
    Agent 依赖注入容器（RunContext.deps）

    说明：与旧版本不同，本版本不再通过 on_tool_call 等回调耦合 UI，
    统一改为事件总线（core.events）。
    """

    config: Config
    cwd: str
    fs: Any = None  # agent_gear.FileSystem 实例（可选）
    todos: list[TodoItem] = field(default_factory=list)
    background_shells: dict[str, tuple[Any, BackgroundShell]] = field(default_factory=dict)
    sub_agents: dict[str, AgentTask] = field(default_factory=dict)
    sub_agent_tasks: dict[str, Any] = field(default_factory=dict)

    # 由运行时注入（避免 tools 直接依赖 Textual）
    event_bus: Any = None  # core.events.EventBus
    ask_user_service: Any = None  # core.services.ask_user.AskUserService
    subagent_service: Any = None  # core.services.subagents.SubAgentService
